@@ -28,7 +28,7 @@
 #*/########################################################################### 
 setConstructorS3("AromaTabularBinaryFile", function(...) {
   this <- extend(GenericTabularFile(..., .verify=FALSE), 
-                                                     "AromaTabularBinaryFile",
+                   c("AromaTabularBinaryFile", uses("FileCacheKeyInterface")),
     "cached:.hdr"=NULL,
     "cached:.ftr"=NULL
   );
@@ -41,7 +41,6 @@ setConstructorS3("AromaTabularBinaryFile", function(...) {
 })
 
 
-
 setMethodS3("as.character", "AromaTabularBinaryFile", function(x, ...) {
   # To please R CMD check
   this <- x;
@@ -1269,21 +1268,24 @@ setMethodS3("summary", "AromaTabularBinaryFile", function(object, ...) {
 })
 
 
-setMethodS3("lapply", "AromaTabularBinaryFile", function(X, FUN, ...) {
+setMethodS3("colApply", "AromaTabularBinaryFile", function(X, FUN, ...) {
   # To please R CMD check
   this <- X;
 
+  # Argument 'FUN':
+  FUN <- match.fun(FUN);
+
   nbrOfColumns <- nbrOfColumns(this);
   res <- base::lapply(seq_len(nbrOfColumns), FUN=function(cc) {
     FUN(this[[cc]], ...);
   });
 
   res;
-})
+}, protected=TRUE)
 
 
 setMethodS3("colStats", "AromaTabularBinaryFile", function(this, FUN, ...) {
-  res <- lapply(this, FUN=FUN, ...);
+  res <- colApply(this, FUN=FUN, ...);
   res <- unlist(res, use.names=FALSE);
   res;
 }, protected=TRUE)
@@ -1324,6 +1326,8 @@ setMethodS3("importFrom", "AromaTabularBinaryFile", function(this, srcFile, ...)
 
 ############################################################################
 # HISTORY:
+# 2012-11-29
+# o Renamed lapply() for AromaTabularBinaryFile to colApply().
 # 2011-09-24
 # o readHeader(), readRawFooter() and writeRawFooter() of 
 #   AromaTabularBinaryFile would try to read non-signed 4-byte integers,
