@@ -8,7 +8,7 @@ setMethodS3("remap", "default", function(x, map, values=NULL, ...) {
 
   # Argument 'values':
   if (is.null(values)) {
-    values <- seq(along=map);
+    values <- seq_along(map);
     mode(values) <- mode;
   }
 
@@ -18,15 +18,15 @@ setMethodS3("remap", "default", function(x, map, values=NULL, ...) {
   }
 
   # Allocate return object
-  y <- vector(mode(values), length(x));
+  y <- vector(mode(values), length=length(x));
   dim(y) <- dim(x);
 
   # Remap
   nbrOfValues <- length(map);
-  for (kk in seq(length=nbrOfValues)) {
+  for (kk in seq_len(nbrOfValues)) {
     idxs <- (x == map[kk]);
-    idxs <- whichVector(idxs);
-    if (length(idxs) > 0) {
+    idxs <- which(idxs);
+    if (length(idxs) > 0L) {
       y[idxs] <- values[kk];
     }
   }
